@@ -2,22 +2,17 @@
 
 __docformat__ = 'restructuredtext'
 
-__version__ = '0.3'
+__version__ = '0.4'
 
-import model
+import models
+import models as model # remain backwards compatibility
 import likelihoods
 import generate
 import utils
-import sandbox
 
-from model import *
+from models import *
 
 import wfpt
-try:
-    import wfpt_switch
-except:
-    pass
-
 
 try:
     import lba
@@ -26,7 +21,4 @@ except ImportError:
 
 from kabuki.utils import load_csv, save_csv
 
-try:
-    from IPython.core.debugger import Tracer; debug_here = Tracer()
-except:
-    def debug_here(): pass
+
