@@ -1,15 +1,14 @@
-
 from __future__ import division
 
-import numpy as np
-from numpy.random import rand
-
-from scipy.stats import uniform, norm
-from copy import copy, deepcopy
 import random
 import kabuki
-
 import hddm
+import numpy as np
+import pandas as pd
+
+from numpy.random import rand
+from scipy.stats import uniform, norm
+from copy import copy, deepcopy
 
 def gen_single_params_set(include=()):
     """Returns a dict of DDM parameters with random values for a singel conditin
@@ -56,7 +55,7 @@ def gen_single_params_set(include=()):
     return params
 
 
-def gen_rand_params(include = (), cond_dict=None, seed=None):
+def gen_rand_params(include=(), cond_dict=None, seed=None):
     """Returns a dict of DDM parameters with random values.
 
         :Optional:
@@ -95,8 +94,6 @@ def gen_rand_params(include = (), cond_dict=None, seed=None):
                 merged_params:
                      a dictionary of parameters that can be used to validate the optimization
                      and learning algorithms.
-
-
     """
 
 
@@ -104,7 +101,7 @@ def gen_rand_params(include = (), cond_dict=None, seed=None):
     if seed is not None:
         np.random.seed(seed)
 
-   #if ther is only a single condition then we can use gen_single_params_set
+    #if there is only a single condition then we can use gen_single_params_set
     if cond_dict is None:
        return gen_single_params_set(include=include)
 
@@ -394,7 +391,7 @@ def _gen_rts_from_cdf(params, samples=1000):
     T = params['st']; a = params['a']
     return hddm.likelihoods.wfpt.ppf(np.random.rand(samples), args=(v, V, a, z, Z, t, T))
 
-def gen_rand_data(params, method='cdf', **kwargs):
+def gen_rand_data(params=None, n_fast_outliers=0, n_slow_outliers=0, **kwargs):
     """Generate simulated RTs with random parameters.
 
        :Optional:
@@ -410,6 +407,10 @@ def gen_rand_data(params, method='cdf', **kwargs):
             parameter values
 
     """
+
+    if params is None:
+        params = gen_rand_params()
+
     from numpy import inf
 
     # set valid param ranges
@@ -421,9 +422,46 @@ def gen_rand_data(params, method='cdf', **kwargs):
               'sz': (0, 1)
     }
 
+#    if 'share_noise' not in kwargs:
+#        kwargs['share_noise'] = set(['a','v','t','st','sz','sv','z'])
+
     # Create RT data
-    data, subj_params = kabuki.generate.gen_rand_data(hddm.likelihoods.Wfpt, params, check_valid_func=hddm.utils.check_params_valid, bounds=bounds, **kwargs)
+    data, subj_params = kabuki.generate.gen_rand_data(hddm.likelihoods.Wfpt, params,
+                                                      check_valid_func=hddm.utils.check_params_valid,
+                                                      bounds=bounds, **kwargs)
     data = kabuki_data_to_hddm_data(data)
 
+    #add outliers
+    seed = kwargs.get('seed', None)
+    data = add_outliers(data, n_fast=n_fast_outliers, n_slow=n_slow_outliers, seed=seed)
+
     return data, subj_params
 
+def add_outliers(data, n_fast, n_slow, seed=None):
+    """add outliers to data. outliers are distrbuted randomly across condition.
+    Input:
+        data - data
+        n_fast/n_slow - numberprobability of fast/slow outliers
+    """
+    data = pd.DataFrame(data)
+    n_outliers = n_fast + n_slow
+    if n_outliers == 0:
+        return data
+
+    if seed is not None:
+        np.random.seed(seed)
+
+    #init outliers DataFrame
+    idx = np.random.permutation(len(data))[:n_outliers]
+    outliers = data.ix[idx].copy()
+
+    #fast outliers
+    outliers.rt[:n_fast] = np.random.rand(n_fast) * (min(abs(data['rt'])) - 0.1001)  + 0.1001
+
+    #slow outliers
+    outliers.rt[n_fast:] = np.random.rand(n_slow) * 2 + max(abs(data['rt']))
+    outliers.response = np.random.randint(0,2,n_outliers)
+
+    #combine data with outliers
+    data = pd.concat((data, outliers), ignore_index=True)
+    return data
